const express = require("express");
const app = express();
const cors = require("cors");
const multer = require("multer");
const path = require("path");
const fs = require("fs");
const videoMap = require("./videoMap.js");
app.use(cors());
app.use(express.json());

const uploadsDir = path.join(__dirname, "uploads");
if (!fs.existsSync(uploadsDir)) {
  fs.mkdirSync(uploadsDir, { recursive: true });
}

// Set up storage engine with Multer
const storage = multer.diskStorage({
  destination: function (req, file, cb) {
    cb(null, uploadsDir);
  },
  filename: function (req, file, cb) {
    cb(null, `${Date.now()}-${file.originalname}`);
  },
});

const upload = multer({ storage: storage });

app.get("/", (req, res) => {
  res
    .status(200)
    .json({ message: "Server is running ", videoSize: videoMap.length });
});


// Another root route
app.get('/another', (req, res) => {
    res.send('Hello, this is another root route!');
});

app.get("/index", (req, res) => {
  res.status(200).json({ message: "response message is fine" });
});

app.post("/video/upload", upload.single("video"), (req, res) => {
  const filePath = path.join(uploadsDir, req.file.filename);
  // Read the existing videoMap file
  const videoMapFile = path.join(__dirname, "videoMap.js");
  let videoMap = [];

  if (fs.existsSync(videoMapFile)) {
    videoMap = require(videoMapFile);
  }

  // Add file path to videoMap array
  videoMap.push(filePath);

  // Write updated videoMap array to videoMap.js
  const updatedVideoMapContent = `const videomap = ${JSON.stringify(
    videoMap,
    null,
    2
  )};\n\nmodule.exports = videomap;\n`;
  fs.writeFileSync(videoMapFile, updatedVideoMapContent, "utf8");
  res
    .status(200)
    .json({ message: "File uploaded successfully", file: req.file });
});

//api for getting the video in the form of Streams

app.get("/video/:index", (req, res) => {
  if (videoMap.length == 0)
    return res.status(404).json({ message: "No video found" });
  let index = req.params.index;
  //   if (index < 0) {
  //     index = (index - 1 + videoMap.length) % videoMap.length;
  //   }
  //   if (index >= videoMap.length) {
  //     index = index % videoMap.length;\

  if (!req.headers.range)
    res.status(404).json({ message: "Range Header not found" });

  const range = req.headers.range;
  const videoPath = videoMap[index];
  const videoSize = fs.statSync(videoPath).size;
  const CHUNK_SIZE = 10 ** 6; // 1MB
  const start = Number(range.replace(/\D/g, ""));
  const end = Math.min(start + CHUNK_SIZE, videoSize - 1);

  const contentLength = end - start + 1;
  const headers = {
    "Content-Range": `bytes ${start}-${end}/${videoSize}`,
    "Accept-Ranges": "bytes",
    "Content-Length": contentLength,
    "Content-Type": "video/mp4",
  };

  res.writeHead(206, headers);
  const videoStream = fs.createReadStream(videoPath, { start, end });
  videoStream.pipe(res);
});

const PORT = process.env.PORT || 3500;

app.listen(PORT, () => {
  console.log(`Server listening on port ${PORT}`);
});
